@@ -77,8 +77,7 @@ class Referencing:
     def __init__(self, *, old=None, new=None):
         if not old and not new:
             raise ValueError(
-                'Must provide either "old" and/or "new" to the referencing'
-                " construct of a trigger"
+                'Must provide either "old" and/or "new" to the referencing construct of a trigger'
             )
 
         self.old = old
@@ -536,7 +535,7 @@ def _cleanup_on_exit(cleanup):
     cleanup()
 
 
-def _ignore_func_name():
+def _ignore_func_name() -> str:
     ignore_func = "_pgtrigger_should_ignore"
     if features.schema():  # pragma: no branch
         ignore_func = f"{utils.quote(features.schema())}.{ignore_func}"
